//! A module benchmarking the `core` backend of `concrete_core`.
use concrete_core::prelude::*;
use criterion::Criterion;

#[rustfmt::skip]
pub fn bench() {
    use crate::generics::*;
    let mut criterion = Criterion::default().configure_from_args();
    cleartext_creation::bench::<CoreEngine, u32, Cleartext32>(&mut criterion);
    cleartext_creation::bench::<CoreEngine, u64, Cleartext64>(&mut criterion);
    cleartext_vector_creation::bench::<CoreEngine, u32, CleartextVector32>(&mut criterion);
    cleartext_vector_creation::bench::<CoreEngine, u64, CleartextVector64>(&mut criterion);
    glwe_ciphertext_decryption::bench::<CoreEngine, GlweSecretKey32, GlweCiphertext32, PlaintextVector32>(&mut criterion);
    glwe_ciphertext_decryption::bench::<CoreEngine, GlweSecretKey64, GlweCiphertext64, PlaintextVector64>(&mut criterion);
    glwe_ciphertext_encryption::bench::<CoreEngine, GlweSecretKey32, PlaintextVector32, GlweCiphertext32>(&mut criterion);
    glwe_ciphertext_encryption::bench::<CoreEngine, GlweSecretKey64, PlaintextVector64, GlweCiphertext64>(&mut criterion);
    glwe_ciphertext_discarding_decryption::bench::<CoreEngine, GlweSecretKey32,  GlweCiphertext32, PlaintextVector32>(&mut criterion);
    glwe_ciphertext_discarding_decryption::bench::<CoreEngine, GlweSecretKey64,  GlweCiphertext64, PlaintextVector64>(&mut criterion);
    glwe_ciphertext_discarding_encryption::bench::<CoreEngine, GlweSecretKey32, PlaintextVector32,  GlweCiphertext32>(&mut criterion);
    glwe_ciphertext_discarding_encryption::bench::<CoreEngine, GlweSecretKey64, PlaintextVector64,  GlweCiphertext64>(&mut criterion);
    glwe_ciphertext_vector_decryption::bench::<CoreEngine, GlweSecretKey32, GlweCiphertextVector32, PlaintextVector32>(&mut criterion);
    glwe_ciphertext_vector_decryption::bench::<CoreEngine, GlweSecretKey64, GlweCiphertextVector64, PlaintextVector64>(&mut criterion);
    glwe_ciphertext_vector_encryption::bench::<CoreEngine, GlweSecretKey32, PlaintextVector32, GlweCiphertextVector32>(&mut criterion);
    glwe_ciphertext_vector_encryption::bench::<CoreEngine, GlweSecretKey64, PlaintextVector64, GlweCiphertextVector64>(&mut criterion);
    glwe_ciphertext_vector_discarding_decryption::bench::<CoreEngine, GlweSecretKey32, GlweCiphertextVector32, PlaintextVector32>(&mut criterion);
    glwe_ciphertext_vector_discarding_decryption::bench::<CoreEngine, GlweSecretKey64, GlweCiphertextVector64, PlaintextVector64>(&mut criterion);
    glwe_ciphertext_vector_discarding_encryption::bench::<CoreEngine, GlweSecretKey32, PlaintextVector32, GlweCiphertextVector32>(&mut criterion);
    glwe_ciphertext_vector_discarding_encryption::bench::<CoreEngine, GlweSecretKey64, PlaintextVector64, GlweCiphertextVector64>(&mut criterion);
    glwe_ciphertext_vector_zero_encryption::bench::<CoreEngine, GlweSecretKey32, GlweCiphertextVector32>(&mut criterion);
    glwe_ciphertext_vector_zero_encryption::bench::<CoreEngine, GlweSecretKey64, GlweCiphertextVector64>(&mut criterion);
    glwe_ciphertext_zero_encryption::bench::<CoreEngine, GlweSecretKey32, GlweCiphertext32>(&mut criterion);
    glwe_ciphertext_zero_encryption::bench::<CoreEngine, GlweSecretKey64, GlweCiphertext64>(&mut criterion);
    glwe_secret_key_creation::bench::<CoreEngine, GlweSecretKey32>(&mut criterion);
    glwe_secret_key_creation::bench::<CoreEngine, GlweSecretKey64>(&mut criterion);
    lwe_bootstrap_key_conversion::bench::<CoreEngine, LweBootstrapKey32, FourierLweBootstrapKey32>(&mut criterion);
    lwe_bootstrap_key_conversion::bench::<CoreEngine, LweBootstrapKey64, FourierLweBootstrapKey64>(&mut criterion);
    lwe_bootstrap_key_creation::bench::<CoreEngine, LweSecretKey32, GlweSecretKey32, LweBootstrapKey32>(&mut criterion);
    lwe_bootstrap_key_creation::bench::<CoreEngine, LweSecretKey64, GlweSecretKey64, LweBootstrapKey64>(&mut criterion);
    lwe_ciphertext_fusing_addition::bench::<CoreEngine, LweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_fusing_addition::bench::<CoreEngine, LweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_bootstrap::bench::<CoreEngine, FourierLweBootstrapKey32, GlweCiphertext32, LweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_bootstrap::bench::<CoreEngine, FourierLweBootstrapKey64, GlweCiphertext64, LweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_fusing_negation::bench::<CoreEngine, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_fusing_negation::bench::<CoreEngine, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_cleartext_fusing_multiplication::bench::<CoreEngine, LweCiphertext32, Cleartext32>(&mut criterion);
    lwe_ciphertext_cleartext_fusing_multiplication::bench::<CoreEngine, LweCiphertext64, Cleartext64>(&mut criterion);
    lwe_ciphertext_cleartext_discarding_multiplication::bench::<CoreEngine, LweCiphertext32, Cleartext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_cleartext_discarding_multiplication::bench::<CoreEngine, LweCiphertext64, Cleartext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_decryption::bench::<CoreEngine, LweSecretKey32, LweCiphertext32, Plaintext32>(&mut criterion);
    lwe_ciphertext_decryption::bench::<CoreEngine, LweSecretKey64, LweCiphertext64, Plaintext64>(&mut criterion);
    lwe_ciphertext_encryption::bench::<CoreEngine, LweSecretKey32, Plaintext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_encryption::bench::<CoreEngine, LweSecretKey64, Plaintext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_addition::bench::<CoreEngine, LweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_addition::bench::<CoreEngine, LweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_vector_discarding_affine_transformation::bench::<CoreEngine, LweCiphertextVector32, CleartextVector32, Plaintext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_vector_discarding_affine_transformation::bench::<CoreEngine, LweCiphertextVector64, CleartextVector64, Plaintext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_decryption::bench::<CoreEngine, LweSecretKey32, LweCiphertext32, Plaintext32>(&mut criterion);
    lwe_ciphertext_discarding_decryption::bench::<CoreEngine, LweSecretKey64, LweCiphertext64, Plaintext64>(&mut criterion);
    lwe_ciphertext_discarding_encryption::bench::<CoreEngine, LweSecretKey32, Plaintext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_encryption::bench::<CoreEngine, LweSecretKey64, Plaintext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_extraction::bench::<CoreEngine, GlweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_extraction::bench::<CoreEngine, GlweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_keyswitch::bench::<CoreEngine, LweKeyswitchKey32, LweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_keyswitch::bench::<CoreEngine, LweKeyswitchKey64, LweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_discarding_negation::bench::<CoreEngine, LweCiphertext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_discarding_negation::bench::<CoreEngine, LweCiphertext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_plaintext_fusing_addition::bench::<CoreEngine, LweCiphertext32, Plaintext32>(&mut criterion);
    lwe_ciphertext_plaintext_fusing_addition::bench::<CoreEngine, LweCiphertext64, Plaintext64>(&mut criterion);
    lwe_ciphertext_plaintext_discarding_addition::bench::<CoreEngine, LweCiphertext32, Plaintext32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_plaintext_discarding_addition::bench::<CoreEngine, LweCiphertext64, Plaintext64, LweCiphertext64>(&mut criterion);
    lwe_ciphertext_vector_decryption::bench::<CoreEngine, LweSecretKey32, LweCiphertextVector32, PlaintextVector32>(&mut criterion);
    lwe_ciphertext_vector_decryption::bench::<CoreEngine, LweSecretKey64, LweCiphertextVector64, PlaintextVector64>(&mut criterion);
    lwe_ciphertext_vector_encryption::bench::<CoreEngine, LweSecretKey32, PlaintextVector32, LweCiphertextVector32>(&mut criterion);
    lwe_ciphertext_vector_encryption::bench::<CoreEngine, LweSecretKey64, PlaintextVector64, LweCiphertextVector64>(&mut criterion);
    lwe_ciphertext_vector_discarding_decryption::bench::<CoreEngine, LweSecretKey32, LweCiphertextVector32, PlaintextVector32>(&mut criterion);
    lwe_ciphertext_vector_discarding_decryption::bench::<CoreEngine, LweSecretKey64, LweCiphertextVector64, PlaintextVector64>(&mut criterion);
    lwe_ciphertext_vector_discarding_encryption::bench::<CoreEngine, LweSecretKey32, PlaintextVector32, LweCiphertextVector32>(&mut criterion);
    lwe_ciphertext_vector_discarding_encryption::bench::<CoreEngine, LweSecretKey64, PlaintextVector64, LweCiphertextVector64>(&mut criterion);
    lwe_ciphertext_vector_zero_encryption::bench::<CoreEngine, LweSecretKey32, LweCiphertextVector32>(&mut criterion);
    lwe_ciphertext_vector_zero_encryption::bench::<CoreEngine, LweSecretKey64, LweCiphertextVector64>(&mut criterion);
    lwe_ciphertext_zero_encryption::bench::<CoreEngine, LweSecretKey32, LweCiphertext32>(&mut criterion);
    lwe_ciphertext_zero_encryption::bench::<CoreEngine, LweSecretKey64, LweCiphertext64>(&mut criterion);
    lwe_keyswitch_key_creation::bench::<CoreEngine, LweSecretKey32, LweSecretKey32, LweKeyswitchKey32>(&mut criterion);
    lwe_keyswitch_key_creation::bench::<CoreEngine, LweSecretKey64, LweSecretKey64, LweKeyswitchKey64>(&mut criterion);
    lwe_secret_key_creation::bench::<CoreEngine, LweSecretKey32>(&mut criterion);
    lwe_secret_key_creation::bench::<CoreEngine, LweSecretKey64>(&mut criterion);
    plaintext_creation::bench::<CoreEngine, u32, Plaintext32>(&mut criterion);
    plaintext_creation::bench::<CoreEngine, u64, Plaintext64>(&mut criterion);
    plaintext_vector_creation::bench::<CoreEngine, u32, PlaintextVector32>(&mut criterion);
    plaintext_vector_creation::bench::<CoreEngine, u64, PlaintextVector64>(&mut criterion);
}
